{% extends 'admin/model/list.html' %}

{% block body %}
    <p>
      This view demonstrated a tree structure, where there's a recursive relationship
      between records on the same table.
    <p>
    <p>
      It also demonstrates how to extend the builtin <em>list.html</em> template, and
      pass your own parameters to the template. E.g. the parameter: <em>foo</em>, which
      has the value <b>{{ foo }}</b>, passed in dynamically from the <em>{{ h.get_current_view().name }}</em> view.
    </p>
    {{ super() }}
{% endblock %}
