# Generated by Django 4.1.3 on 2022-12-04 21:22

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = []

    operations = [
        migrations.CreateModel(
            name="Upload",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("name", models.CharField(max_length=255, unique=True)),
                ("description", models.TextField(blank=True, null=True)),
                ("file", models.FileField(upload_to="")),
            ],
            options={
                "verbose_name": "Upload",
                "verbose_name_plural": "Upload",
            },
        ),
    ]
